@@ -17,6 +17,7 @@ from neutron_lib.api.definitions import auto_allocated_topology
 from neutron_lib.api.definitions import availability_zone
 from neutron_lib.api.definitions import bgpvpn
 from neutron_lib.api.definitions import bgpvpn_routes_control
+from neutron_lib.api.definitions import bgpvpn_vni
 from neutron_lib.api.definitions import data_plane_status
 from neutron_lib.api.definitions import default_subnetpools
 from neutron_lib.api.definitions import dns
@@ -29,6 +30,7 @@ from neutron_lib.api.definitions import fip64
 from neutron_lib.api.definitions import firewall
 from neutron_lib.api.definitions import firewall_v2
 from neutron_lib.api.definitions import firewallrouterinsertion
+from neutron_lib.api.definitions import flavors
 from neutron_lib.api.definitions import ip_allocation
 from neutron_lib.api.definitions import l2_adjacency
 from neutron_lib.api.definitions import l3
@@ -68,6 +70,7 @@ _ALL_API_DEFINITIONS = {
     availability_zone,
     bgpvpn,
     bgpvpn_routes_control,
+    bgpvpn_vni,
     data_plane_status,
     default_subnetpools,
     dns,
@@ -80,6 +83,7 @@ _ALL_API_DEFINITIONS = {
     firewall,
     firewall_v2,
     firewallrouterinsertion,
+    flavors,
     ip_allocation,
     l2_adjacency,
     l3,
